package PureMessage::AtomicDB_File;
use strict;
use warnings;

use Fcntl qw(O_RDWR O_CREAT);
use Carp;
use DB_File qw($DB_HASH R_FIRST R_NEXT);
use ActiveState::File::Atomic;

sub new {
    my $p = shift;
    my $class = ref($p) || $p;
    my %opt = @_;

    croak "Missing required parameter 'target'" unless $opt{target};
    croak "Missing required parameter 'temp'"   unless $opt{temp};
    
    # XXX Copied from PM::QDigest::DB
    #
    # Get an exclusive lock on the database. Print it out to a temporary file,
    # and then close the tempfile and open *that* with DB_File. If anything
    # goes wrong, we'll lose the changes, but we won't corrupt the original
    # file. You must call $self->save() to commit the changes.
    my $exists = -f $opt{target} && -s $opt{target};
    my $lock = ActiveState::File::Atomic->new(
	$opt{target}, writable => 1, create => 1
    );
    my $temp = "$opt{temp}.$$";
    if ($exists) {
	open (my $ORIG, $opt{target}) or die "can't open $opt{target}: $!";
	open (my $CP, '>', $temp) or die "can't write $temp: $!";
	binmode($ORIG);
	binmode($CP);
	local $/ = \4096;
	while (<$ORIG>) {
	    print $CP $_;
	}
	close $CP or die "can't write $temp: $!";
	close $ORIG;
    }

    my $db = DB_File->new($temp, O_RDWR|O_CREAT, 0666, $DB_HASH)
	or die "can't open database '$temp': $!";
    my %data;
    $data{db_file} = $db;
    $data{db_lock} = $lock;
    $data{db_temp} = $temp;
    
    return bless \%data, $class;
}

sub put {
    my ($self, $key, $val, $flags) = @_;

    # XXX DB_File has this anoying behavior where it prints a warning if
    # you pass an undefined value for the $flags parameter. 
    if (defined $flags) {
        return $self->{db_file}->put($key, $val, $flags);
    }
    else {
	return $self->{db_file}->put($key, $val);
    }

}

sub get {
    my $self = shift;
    my $key = shift;

    # XXX DB_File has this anoying behavior where it prints a warning if
    # you pass an undefined value for the $flags parameter. 
    if (defined $_[1]) {

	# DB_File returns the found value using the value parameter. Maintain
	# that behavior in this interface too.
        return $self->{db_file}->get($key, $_[0], $_[1]);
    }
    else {
	return $self->{db_file}->get($key, $_[0]);
    }
}

sub save {
    my $self = shift;
    $self->{db_file}->sync;
    delete $self->{db_file};
    $self->{db_lock}->commit_file($self->{db_temp});
    delete $self->{db_lock};
    unlink $self->{db_temp};
    delete $self->{db_temp};
}

sub DESTROY {
    my $self = shift;
    delete $self->{db_file};
    unlink $self->{db_temp} if defined $self->{db_temp};
}

sub dump {
    my ($self, $cb) = @_;
    my ($key, $val, $tmp);
    my $db = $self->{db_file};
    
    for ($tmp = $db->seq($key, $val, R_FIRST);
	 $tmp == 0;
	 $tmp = $db->seq($key, $val, R_NEXT))
     {
	 &$cb($key, $val);
     }
}

1;


