package PureMessage::List::Base::cdbfile;
use strict;
use base qw(PureMessage::List::Items PureMessage::List::Base::file);
use CDB_File;
use Carp qw(croak);

sub new {
    my($class, $file, %conf) = @_;
    $conf{file} = $file = _source2cdbfile($class->_abs_file($file));
    croak qq{fatal: $file: cdbfile resources only support "is" and "mail-parts" match types, not "$conf{match_type}"}
        unless(!$conf{match_type} or $conf{match_type} eq 'is' or $conf{match_type} eq 'mail-parts');
    
    $conf{ascii_casemap} = !$conf{comparator} ||
	                   $conf{comparator} eq "i;ascii-casemap";

    my $self = bless \%conf, $class;
    # testload the cdb
    $self->refresh;
    $self;
}

sub refresh {
    my $self = shift;
    my $file = $self->{file};
    my($fsize, $mtime) = (stat $file)[7,9];
    return 0 if $self->{mtime} and $fsize == $self->{fsize} && $mtime == $self->{mtime};
    $self->{cdb} = $self->_new_cdb_handle($file);
    $self->{matcher} = $self->_generate_matcher($self->{cdb});
    @{$self}{"fsize", "mtime"} = (stat $file)[7,9];
    return 1;
}

sub _new_cdb_handle {
    my($self, $file) = @_;
    my $cdbh = CDB_File->TIEHASH($file);
    croak("Can't open cdbfile '$file': $!")
        unless($cdbh && ref($cdbh));
    return $cdbh;
}

sub _source2cdbfile {
    return shift() . ".cdb";
}

1;
