package PureMessage::List::Base::file;

use strict;

use Carp qw(croak);
use PureMessage::Version qw($PREFIX);
use PureMessage::List::Utils 'list_absfile';

sub _abs_file {
    my $class = shift;
    my $file = shift;
    return list_absfile($file);
}

sub new {
    my($class, $file, %conf) = @_;
    $conf{file} = $file = $class->_abs_file($file);
    open(my $f, $file) || croak("Can't open '$file': $!");
    @conf{"fsize", "mtime"} = (stat $f)[7,9];
    $conf{items} = $class->_parse_fh($f, $file);
    return $class->_new_items(%conf);
}

sub file { shift->{file} }

sub refresh {
    my $self = shift;
    my $file = $self->file;
    my($fsize, $mtime) = (stat $file)[7,9];
    return 0 if $fsize == $self->{fsize} && $mtime == $self->{mtime};

    open(my $f, $file) || croak("Can't open '$file': $!");
    @{$self}{"fsize", "mtime"} = (stat $f)[7,9];
    $self->items($self->_parse_fh($f, $file));
    return 1;
}

1;
