package PureMessage::List::Utils;

# Utility functions used by the implementation of Lists and Maps

use strict;
use base 'Exporter';
use PureMessage::Version '$PREFIX';
use constant SUB_KEY_DEPTH => 2;
use File::Path qw(mkpath);
our @EXPORT_OK = qw(_str list_absfile list_key2file list_file2key path_rename mkpath_dotdot);

# convienence function to call mkpath() one level above a specified pathname,
# to make sure all directories leading up to a file exist.
sub mkpath_dotdot {
    my $path = shift;
    $path =~ s{/$}{};
    $path =~ s{/[^/]+?$}{};

    return 0 if -d($path);

    eval { mkpath($path); };
    return $@ ? $@ : 0;
}

# rename a file, implicitly making sure the destination directory
# of the new file exists.
sub path_rename {
    my($old_path, $new_path, $rmdir) = @_;

    return unless -e $old_path;
    return undef if mkpath_dotdot($new_path);

    my $rv = rename($old_path, $new_path);

    if($rv && $rmdir) {
        my($old_dir) = $old_path;
        $old_dir =~ s{/$}{};
        $old_dir =~ s{/.+$}{};
        rmdir($old_dir);
    }

    return defined($rv) ? $rv : undef;    
}

sub _str {
    my($str, $binary) = @_;
    if ($binary) {
	$str = join("", map sprintf("\\x%02x", $_), unpack("C*", $str));
    }
    else {
	$str =~ s/\\/\\\\/g;
	$str =~ s/@/\\@/g;
	$str =~ s/\$/\\\$/g;
	$str =~ s/\"/\\\"/g;
	$str =~ s/\r/\\r/g;
	$str =~ s/\n/\\n/g;
	$str =~ s/\t/\\t/g;
	$str =~ s/([^\x20-\x7e])/sprintf("\\x%02x", ord($1))/ge;
    }
    return qq("$str");
}

sub list_absfile {
    my $file = shift;
    $file = "$PREFIX/etc/$file" unless $file =~ m,^/, or $file =~ m,^$PREFIX/etc,;  # absolute
    $file;
}

# a quoted-printable-like codec to ensure filesystem-friendly keys
sub list_key2file {
    my $key = shift;
    my $path = shift;

    # $key is the base key we're looking for (zaphod)
    $key =~ s/([^\w\@\-\.])/sprintf("=%02X", ord($1))/eg;

    # yes, this will split up the escaped hex values above,
    # which means that any key that begins with an "illegal"
    # character will be filed in a sub-directory of "=".
    my $sub_key = substr($key, 0, SUB_KEY_DEPTH);
    
    if(length($sub_key) < SUB_KEY_DEPTH) {
        $sub_key .= "_" x (SUB_KEY_DEPTH - length($sub_key));
    }
    
    # $full_key is the key with path levels added (z/a/zaphod)
    my $full_key = join('/',
        (map { substr($sub_key, $_, 1) } (0 .. length($sub_key) - 1)),
        $key
    );

    if(wantarray) {
        # in array context, return both the full keys' pathname,
        # and the original not-deepened pathname for auto-upgrading
        return(
            (defined $path ? "$path/$full_key" : $full_key),
            (defined $path ? "$path/$key" : $key)
        );
    } else {
        return defined $path ? "$path/$full_key" : $full_key;
    }
}

sub list_file2key {
    my $file = shift;
    $file =~ s,.*/,,;
    $file =~ s/=([\da-fA-F]{2})/pack("C", hex($1))/ge;
    return $file;
}

1;
