package PureMessage::List::file;

use strict;
use Carp qw(carp);
use base qw(PureMessage::List::Base::file PureMessage::List::Items);
use ActiveState::Handy;
use ActiveState::File::Atomic;

sub _parse_fh {
    my($class, $f, $filename) = @_;
    my @items;
    local $_;
    while (<$f>) {
	next if /^\s*\#/;
	s/^\s+//;
	s/\s+\z//;
	next unless length;
	if (s/^\"//) {
	    # quoted value
	    if (/^((?:[^\"\\]|\\[\"\\])*)\"\s*(\#.*)?\z/) {
                print '$1 = ', $1, "\n";
	        $_ = $1;
	        s/\\(.)/$1/g;
            }
            else {
		carp("Illegal quoted value at $filename line $.");
                next;
            }
	}
	else {
	    s/\s+\#.*//;  # trailing comment
	}
	push(@items, $_);
    }

    return \@items;
}

sub cat {
    my ($self) = @_;
    return ActiveState::Handy::cat($self->file);
}

sub replace_content {
    my ($self, $new_content) = @_;

    my $asfa = ActiveState::File::Atomic->new($self->file,
                                              writable => 1);
    $asfa->commit_string($new_content);
}

1;
