package PureMessage::List::ldap;

use strict;
use base qw(PureMessage::List::Base::ldap PureMessage::List);
use Carp qw(croak);

sub match {
    my $self = shift;
    my $match = shift;
    my $f = $self->{args}{match};
    $match =~ s/([\\\(\)\*\0-\37])/sprintf("\\%02x",ord($1))/sge;
    if (($f =~ /\%[d|u]/) && ($match =~ /@/)) {
        (my $user, my $domain) = split(/@/,$match);
        $f =~ s/%u/$user/g;
        $f =~ s/%d/$domain/g;
    }
    $f =~ s/%s/$match/g;
    my $m = $self->ldap_search( base => $self->{args}{base},
                                filter => $f,
                                scope => $self->{args}{scope},
                              );
    if ($m->is_error) {
        # Store error info in variables--after calling ldap_clear_connection(),
        # it would be rather sketchy to access the Net::LDAP::Message ($m)
        # returned from the search.
        my $error_name = $m->error_name;
        my $code = $m->code;
        my $error = $m->error;
        eval { $self->ldap_clear_connection(); };
        croak "Error performing LDAP search: $error_name($code): $error";
    }
    my $items = 0;
    for my $e ($m->all_entries) {
	$e->dump if $self->{verbose};
	++$items;
    }
    return $items;
}

1;
