#!perl
# Common functions and methods for ListData object manipulation
#
# Not included here: commit() and enumerate_items(); different ListData
# types need different approaches to these.
#

package PureMessage::ListData::Base;

use strict;
use warnings;
use Exporter;
use base q(Exporter);
use vars qw(@EXPORT);

sub _parse_line ($);
sub _to_add ($$$);

@EXPORT = qw(_parse_line _to_add);

return 1;

## Functions

sub _as_item {
    $_[1];
}

sub _parse_line ($) {
    local $_ = shift;
    return "" if /^\s*\#/;
    s/^\s+//;
    s/\s+\z//;
    s/\s+\#.*// unless /^\"/; # trailing comment
    $_;
}

sub _to_add ($$$) {
    my $to_add = shift;
    my $to_add_order = shift;
    my $cb = shift;
    my $affected = 0;
    
    foreach my $item (@$to_add_order) {
        next unless exists $to_add->{"$item"};
        $cb->($item);
        $to_add->{"$item"}--;
        delete $to_add->{"$item"} if $to_add->{"$item"} < 1;
        $affected++;
    }
    return $affected;
}

## Methods

sub parse {
    _parse_line($_[1]);
}

sub add_items {
    my $self = shift;
    my @items = map { $self->_as_item($_) } @_;

    $self->{to_add}{"$_"}++ foreach @items;
    push @{$self->{to_add_order}}, $_ foreach @items;
}

sub remove_items {
    my $self = shift;
    my @items = map { $self->_as_item($_) } @_;
    my $i;
    
    foreach $i (@items) {
        $self->{to_remove}{"$i"} = $i;
        if($self->{to_add}{"$i"}) {
            my $p = (grep($self->{to_add_order}->[$_] eq $i, $[ .. $#{$self->{to_add_order}}))[0];
            # match string "0" as well
            if(defined $p && length $p) {
                splice(@{$self->{to_add_order}}, $p, 1);
            }
            
            delete $self->{to_add}{"$i"};
        }
    }
}

sub clear {
    my $self = shift;
    $self->{clear} = 1;
    delete $self->{to_add};
    delete $self->{to_remove};
}

sub delete_source {
    my $self = shift;
    $self->{delete_source} = 1;
}

sub _modified {
    my $self = shift;
    return (exists $self->{clear} or exists $self->{to_add} or exists $self->{to_remove} or exists $self->{delete_source});
}

sub is_readonly () {0};
