#!/usr/bin/perl

package PureMessage::ListData::Queue;
use PureMessage::ListData;

use strict;

sub COMMANDS () {('add', 'remove', 'clear')};

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $prefix = shift;
    die "Missing prefix parameter" unless defined $prefix;
    my %opts = @_;
   
    if ($opts{create}) {
        -d $prefix or mkdir $prefix or die "Couldn't mkdir $prefix: $!";
    }
    else {
        -d $prefix or die "Listdata queue does not exist: $prefix";
    }

    my $inqueue = "$prefix/inqueue";
    my $outqueue = "$prefix/outqueue";
    my $fh;
    
    my $self =  bless \%opts, $class;
    $self->{prefix} = $prefix;
    $self->{inqueue} = $inqueue;
    $self->{outqueue} = $outqueue;
    $self->{logger} = $opts{logger};
    $self;
}

sub process {
    my $self = shift;
    my $inqueue = $self->{inqueue};
    my $outqueue = $self->{outqueue};
    
    if (-f $outqueue) {
        $self->_log("WARNING", "Found old in-process queue (from a killed process perhaps?).  Processing old entries first\n");
        _read_diffs($outqueue, 1, @_); 
        unlink $outqueue or die "Couldn't unlink $outqueue: $!";
    }
    
    return unless -f $inqueue;
    close delete $self->{inqueue_fh} if $self->{inqueue_fh};
    rename $inqueue, $outqueue or die "Couldn't move $inqueue to $outqueue: $!";
    _read_diffs($outqueue, 1, @_);    
    unlink $outqueue or die "Couldn't unlink $outqueue: $!";
}

sub add {
    my ($self, $command, $list, $key, @items) = @_;
    $list = $self->{list} if defined $self->{list} and not defined $list;
    $key = $self->{key} if defined $self->{key} and not defined $key;
    die "Missing list parameter" unless defined $list;
    unless ($self->{inqueue_fh}) {
        my $inqueue = $self->{inqueue};
        open $self->{inqueue_fh}, ">>", $inqueue or die "Couldn't open $inqueue: $!";
    }
    my $fh = $self->{inqueue_fh};
    print $fh PureMessage::ListData::make_diff($command, $list, $key, @items), "\n" or die "Couldn't print to $self->{inqueue}: $!";
}

sub _read_diffs {
    my ($queue, $decode, $cb) = @_;
    open FH, $queue or die "$queue: $!";
    while (<FH>) {
        next if /^[#\n]/;
        chomp;
        if ($decode) {
            $cb->(PureMessage::ListData::parse_diff($_));
        }
        else {
            $cb->($_);
        }
    }
    close FH;
}

sub read {
    my $self = shift;
    return unless -f $self->{inqueue};
    close delete $self->{inqueue_fh} if $self->{inqueue_fh};
    _read_diffs($self->{inqueue}, 1, @_);
}

sub read_diffs {
    my $self = shift;
    return unless -f $self->{inqueue};
    close delete $self->{inqueue_fh} if $self->{inqueue_fh};
    _read_diffs($self->{inqueue}, 0, @_);
}

sub cleanup {
    my $self = shift;
    close $self->{inqueue_fh} if $self->{inqueue_fh};
    unlink $self->{inqueue};
    unlink $self->{outqueue};
    rmdir $self->{prefix} or die "Couldn't unlink $self->{prefix}: $!";
}

sub is_empty {
    my $self = shift;
    close $self->{inqueue_fh} if $self->{inqueue_fh};
    foreach my $q ($self->{inqueue}, $self->{outqueue}) {
        if (-e $q) {
            open my $fh, $q or die "$q: $!";
            while (<$fh>) {
                return 0 if /^\s*[^\s#]/;
            }
        }
    }
    return 1;
}

sub _log {
    my $self = shift;
    $self->{logger}->log(@_) if $self->{logger};
}

1;
