#!/usr/bin/perl

package PureMessage::ListData::SyncQueue;

use strict;
use PureMessage::Version '$PREFIX';
use PureMessage::ListData::Queue;
use PureMessage::Logger;
use ActiveState::File::Atomic;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %opts = @_;
    
    my $prefix = "$PREFIX/var/listsync";
   
    if ($opts{create}) {
        -d $prefix or mkdir $prefix or die "Couldn't mkdir $prefix: $!";
    }
    else {
        -d $prefix or die "Listsync queue at $prefix not initialized";
    }
    
    my $queue = PureMessage::ListData::Queue->new($prefix, %opts);
    return bless {
        prefix => $prefix,
        queue => $queue,
        logger => $opts{logger} || PureMessage::Logger->new(),
    }, $class;
}

sub add {
    shift->{queue}->add(@_);
}

sub is_empty {
    shift->{queue}->is_empty();
}

sub process_queues {
    my $self = shift;
    my $key_or_keys = shift;
    my $cb = shift;
    my @keys = ref $key_or_keys ? @$key_or_keys : $key_or_keys;
    my $qpath = $self->{prefix} . "/queue";
    my %queues;
    $self->_lock;
    eval {
        foreach my $key (@keys) {
            $queues{$key} = PureMessage::ListData::Queue->new("$qpath.$key", create=>1, add=>1, logger=>$self->{logger});
        }
        if ($self->{queue}->is_empty) {
            $self->_log('DEBUG', 'Main list queue is empty');
        }
        else {
            $self->{queue}->process(
                sub {
                    foreach my $key (@keys) {
                        $queues{$key}->add(@_);
                    }
                }
            );
        }
        foreach my $key (@keys) {
            $cb->($key, $queues{$key});
        }
    };
    $self->_unlock;
    die $@ if $@;
}

sub _lock {
    my $self = shift;
    my $lock = $self->{prefix}."/lock";
    eval {
        $self->{lock} = ActiveState::File::Atomic->new($lock, create=>1, writable=>1, timeout=>1);
    };
    if ($@) {
        if ($@ =~ /^Can't lock file/) {
            die <<EOT; 
Could not acquire lock on the list synchronization queue - it is in use
by another process.  If you think the lock might be owned by a stale
process, try killing the process and/or deleting '$lock'.
EOT
        }
        else {
            die $@;
        }
    }
}
sub _unlock {
    my $self = shift;
    # lock is given up on File::Atomic object destruction
    delete $self->{lock};
    unlink $self->{prefix}."/lock";
}
sub _log {
    shift->{logger}->log(@_);
}

1;
