package PureMessage::Map::Items;

use strict;
use base qw(PureMessage::Map Exporter);
use Carp qw(cluck croak);

use  PureMessage::List::Utils qw(_str);

our $DEBUG = $ENV{PMX_MAP_DEBUG};
our @MatchMapDefault = qw(is mail mail-parts);

our @EXPORT_OK = qw(@MatchMapDefault);



sub _generate_transformer {
    my($self, $tie) = @_;

    my $match_type = $self->{match_type} ||= "is";
    my $ascii_casemap = ($self->{comparator} || "") eq "i;ascii-casemap";
    
    my @items = @{$self->{items}} unless($tie);

    my @trans_code;
    if ($match_type eq "is" || $match_type eq "mail-parts") {
        if($tie) {
            push(@trans_code, "my \%trans;\n");
            push(@trans_code, "tie \%trans, 'PureMessage::List::Items::retie', \$tie;\n");
        } else {
            push(@trans_code, "my %trans = (\n");
            my %seen;
            for (@items) {
                my $key = $_->[0];
                $key =~ tr/A-Z/a-z/ if $ascii_casemap;
                next if $seen{$key}++;
                push(@trans_code, "  ", _str($key), " => [");
                my @values = @$_;
                shift(@values);
                push(@trans_code, join(", ", map _str($_), @values));
                push(@trans_code, "],\n");
            }
            push(@trans_code, ");\n");
        }
    }

    push(@trans_code, "sub {\n");
    push(@trans_code, "  my \@res;\n");

    my $orig;
    if ($ascii_casemap) {
	push(@trans_code, "  local \$_;\n");
	push(@trans_code, "  while (\@_) {\n");
	push(@trans_code, "    my \$orig = \$_ = shift;\n");
	push(@trans_code, "    tr/A-Z/a-z/;\n");
	$orig = '$orig';
    }
    elsif ($match_type eq "mail") {
	push(@trans_code, "  for my \$orig (\@_) {\n");
	$orig = '$orig';
    }
    else {
	push(@trans_code, "  for (\@_) {\n");
	$orig = '$_';
    }

    if ($match_type eq "is") {
	push(@trans_code, "    my \$m = \$trans{\$_};\n");
	push(@trans_code, "    push(\@res, \$m ? \@\$m : $orig);\n");
    } elsif($match_type eq "mail-parts") {
        push(@trans_code, "    my \$at_symbol = index(\$_, '\@');\n");
        push(@trans_code, "    my \$m = \$trans{\$_} || \$trans{substr(\$_, \$at_symbol)} || \$trans{substr(\$_, 0, \$at_symbol + 1)};\n");
        push(@trans_code, "    push(\@res, \$m ? \@\$m : $orig);\n");
    } elsif($tie) {
        croak q{tie()d maps only support "is" or "mail-parts" match types};
    }
    elsif ($match_type eq "mail") {
	push(@trans_code, "    local \$_ = $orig;\n");
	push(@trans_code, "    if (0) {\n");
	push(@trans_code, "      # dummy\n");
	push(@trans_code, "    }\n");
	for (@items) {
	    my($key, @values) = @$_;
	    for ($key) {
		$_ = quotemeta($_);
		s/(?<!\\)\\\?/([^@.])/g;
		s/(?<!\\)\\\*/([^@.]*)/g;
		s/\\\\\\([*?])/\\$1/g;
		$_ .= '\z' unless /\@\z/;
		if (/\\\@/) {
		    $_ = "^$_" unless /^\\\@/;
		}
		else {
		    $_ = "(?<=[\@.])$_" unless /^\\/;
		}
		s/(?:\(\[\^\@\.\]\*\)){2,}/([^@]*)/g;
	    }
	    my $num_wildcards = 0;
	    $num_wildcards++ while $key =~ /\(\[/gc;
	    if (@values == 1) {
		my $to = $values[0];
		$to =~ s/([@\$\\\/])/\\$1/g;
		for (1..$num_wildcards) {
		    $to =~ s/(\?|\*+)/\$$_/;
		}
		push(@trans_code, "    elsif (s/$key/$to/i) {\n");
		push(@trans_code, "      push(\@res, \$_)\n");
	    }
	    elsif (@values == 0) {
		push(@trans_code, "    elsif (/$key/i) {\n");
		push(@trans_code, "      # empty mapping\n");
	    }
	    else {
		push(@trans_code, "    elsif (s/$key/\\0\\0/i) {\n");
		if ($num_wildcards) {
		    push(@trans_code, "      my \@w = (", join(", ", map "\$$_", 1 .. $num_wildcards), ");\n");
		}
		push(@trans_code, "      for my \$r (", join(", ", map _str($_), @values), ") {\n");
		push(@trans_code, "        my \$v = \$_;\n");
		push(@trans_code, "        \$v =~ s/\\0\\0/\$r/ || die 'Assert';\n");
		if ($num_wildcards) {
		    push(@trans_code, "        for my \$w (\@w) {\n");
		    push(@trans_code, "          \$v =~ s/(\\?|\\*+)/\$w/;\n");
		    push(@trans_code, "        }\n");
		}
		push(@trans_code, "        push(\@res, \$v);\n");
		push(@trans_code, "      }\n");
	    }
	    push(@trans_code, "    }\n");
	}
	push(@trans_code, "    else {\n");
	push(@trans_code, "      push(\@res, \$_);\n");
	push(@trans_code, "    }\n");
    }
    else {
	die "Unsupported match_type '$match_type'";
    }

    push(@trans_code, "  }\n  return wantarray ? \@res : \$res[0];\n}\n");

    if ($DEBUG) {
	my $id = $self->{id} || "?";
	print "-- map '$id' ($match_type) --\n", @trans_code, "\n";
    }

    # compile code
    my $transformer = eval join("", @trans_code);
    die $@ if $@;

    return $transformer;
}

sub _new_items {
    new(@_);
}

sub new {
    my($class, %conf) = @_;

    my $self = bless \%conf, $class;
    $self->{transformer} = _generate_transformer($self);
    $self;
}

sub items {
    my $self = shift;
    my $old = $self->{items};
    if (@_) {
	$self->{items} = [(@_ == 1 && ref($_[0]) eq "ARRAY") ? @{$_[0]} : @_];
	$self->{transformer} = _generate_transformer($self);
    }
    @$old;
}

sub transform {
    my $self = shift;
    $self->{transformer}->(@_);
}

sub transformer {
    shift->{transformer};
}

1;
