#!perl

package PureMessage::MapData::Entry::Item;

use strict;
use warnings;
use ActiveState::UTF8 qw(encode_utf8);

use overload (
    '""'        => \&value,
    'cmp'       => \&cmp_value,
    fallback    => 1
);

return 1;

sub new {
    my($class, $scalar) = @_;
    $scalar = '' if(!defined($scalar));
    bless \$scalar, $class;    
}
    
sub as_string {
    my $self = shift;
    local $_ = $self->value;
    
    if (/[\s\#\"\:]|^$/) {
        s/([\\\"])/\\$1/g;
        $_ = qq("$_");
    }
    return $_;
}

sub cmp_value {
    my($a, $b) = @_;
    if(ref($b) && ref($b) eq ref($a)) {
        $a->value cmp $b->value;
    } else {
        $a->value cmp "$b";
    }
}

sub value {
    my $self = shift;
    my $value = $$self;
    if($_[0]) {
        $$self = $_[0];
    }
    $value;
}


=pod

=head1 NAME

PureMessage::MapData::Entry::Item - one "column" in one "row" of a Map.

=head1 SYNOPSIS

my $item = PureMessage::MapData::Entry::Item->new("a:b:c@d.e.f.com");

print "stuff:\t$item\n";

$sth->execute($item->value);

=head1 DESCRIPTION

This module is primarily used by PureMessage::MapData::Entry. In most cases
you will want to use that module instead of using this one directly.

=head1 CONSTRUCTOR

PureMessage::MapData::Entry::Item->new($value)

$value should be the actual value of the item, not escaped in any way.

=head1 METHOD

$item->value([$new_value])

Returns the present unescaped value of the item, and optionally assigns
a new value to it.

=head1 OPERATOR

"$item"

When an Item is used as part of a string, it is returned as the value of
the item, with mapfile escaping applied to it.

=head1 SEE ALSO

L<PureMessage::MapData::Entry>

=cut
