package PureMessage::MapFile;

use strict;
use base qw(PureMessage::ListFile);
use ActiveState::UTF8 qw(decode_utf8);
use PureMessage::MapData::Entry;
use PureMessage::MapData::Entry::Item;

sub _parse_line {
    PureMessage::MapData::Entry->parse($_[1]);
}

sub _entry {
    my $e = $_[1];
    return $e if(UNIVERSAL::isa($e, "PureMessage::MapData::Entry"));
    return PureMessage::MapData::Entry->new(@$e) if ref($e) eq "ARRAY";
    return PureMessage::MapData::Entry->new($e);
}

sub _item {
    my $e = $_[1];
    return $e if(UNIVERSAL::isa($e, "PureMessage::MapData::Entry::Item"));
    return PureMessage::MapData::Entry::Item->new($e);
}

sub _format_entry {
    my($self, $e) = @_;
    join("\n", $self->_entry($e));
}

sub _format_item {
    my($self, $item) = @_;
    $self->_item($item)->as_string;
}


1;

__END__

=head1 NAME

PureMessage::MapFile - Mapfile edit interface

=head1 SYNOPSIS

  use PureMessage::MapFile;

  my $list = PureMessage::MapFile->new("notifications", writable => 1);
  $list->append(['foo@bar.com' => 'bar@foo.org']);
  $list->commit;

=head1 DESCRIPTION

The C<PureMessage::MapFile> class can be used to examine or modify the
entries and comments found in a map file.  It is a subclass of
C<PureMessage::ListFile> and provides the same interface as that class.

What is different is the format of the entries.  Entries returned by
the entry() or the entries() methods are objects with a left() and a
right() method.  The left() method returns an array reference of the
source mapping, i.e. what is to the left of the colon in the file format.
The right() method returns an array reference of the destination
mapping, i.e. what is to the right of the colon.  The entry objects also
stringify as "left: right".

For the append() method entries can be provided as plain array
references like:

  ["left" => "right"]
  [["left"] => ["right"]]

The second form allows multiple items on each side:

  [["a", "b"] => "foo"]

=head1 SEE ALSO

L<PureMessage::ListFile>, L<PureMessage::Map>
