package PureMessage::Mbox;

use strict;
use warnings;
use base 'Exporter';
use Carp;
use PureMessage::Mbox::Writer;

our @EXPORT_OK = qw(mbox_append);

sub mbox_append {
    my $file = shift;
    my $fh_or_str = shift;
    my %opts = @_;
    
    open my $fh, ">>", $file or croak "Could not open '$file' for writing: $!";
    my $mbox = PureMessage::Mbox::Writer->new($fh, block_size=>$opts{block_size}, eol=>$opts{eol});
    
    $mbox->from(env_sender=>$opts{env_sender}, date=>$opts{date}, more_info=>$opts{more_info});
    
    if (ref $fh_or_str) {
        $mbox->data_fh($fh_or_str);
    }
    else {
        $mbox->data($fh_or_str);
    }
    $mbox->eom();
    close $fh;
}

1;


__END__

=head1 NAME

PureMessage::Mbox

=head1 SYNOPSIS

Utility functions to manipulate mbox files.

=head1 DESCRIPTION

=over 4

=item B<mbox_append($filename, $fh_or_str, %opts)>

Append an RFC2822 message to an mbox file.  This is a convenience method
that uses PureMessage::Mbox::Writer to do the actual work.

Parameters:

=over 4

=item $filename

The name of the mbox file to append to.

=item $fh_or_str

RFC2822 message data to append. Can be a filehandle (a blessed ref that
can read() or a globref) or a string.  

See L<PureMessage::Mbox::Writer> for descriptions of the following options.

These options are passed to PureMessage::Mbox::Writer->new:

=over 4

=item $opts{eol}

=item $opts{block_size}

=back

These options are passed to PureMessage::Mbox::Writer->from:

=over 4

=item $opts{env_sender}

=item $opts{date}

=item $opts{more_info}

=back

=back

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2000-2009 Sophos Group
All Rights Reserved.

=head1 SEE ALSO

For a description of the mbox format see: 

http://www.qmail.org/qmail-manual-html/man5/mbox.html

L<PureMessage::Mbox::Writer>

