package PureMessage::MultiList;

use strict;
use base 'PureMessage::List::Base';

sub _kind { "multilist" }

1;

__END__

=head1 NAME

PureMessage::MultiList - multidimensional lists

=head1 SYNOPSIS

 use PureMessage::MultiList;

=head1 DESCRIPTION

The C<PureMessage::MultiList> class provides a multidimensional list object,
meant for PureMessage end-user whitelists, blacklists and preferences.  It can
be understood as a "list of lists".  It does not do any matching itself, it
just knows about a set of lists that are indexed by some unique key.

PureMessage::MultiList supports the PureMessage::List::Base interface (like
PureMessage::List and PureMessage::Map), and provides some specific methods:

=over 4

=item B<find_list(key=E<gt>$key, %opts)>

Retrieve a list corresponding to the specified key.  This method will croak if
the key option is not specified.

If the list does not exist for that user, find_list returns undef by default.
If, however, edit_mode=>1 is specified, find_list in this situation will
return a PureMessage::List::null object, which won't match anything but can be
used to autovivify and edit the list via the data method (see
L<PureMessage::ListData>).  This mechanism is used because lists will normally
croak if they cannot find their data source.  If create_mode=>1 is specified,
find_list will touch the sublist file, and return a proper
PureMessage::List::file object.

The retrieved list object is cached for future access.

=item B<enumerate_keys(sub { my $key = shift })>

Call a sub for each of the keys/users.

=item B<refresh()>

Refresh all the lists in the cache.

=back

=head1 AUTHOR

ActiveState Corporation

=head1 COPYRIGHT

Copyright (C) 2003, ActiveState Corporation.
All Rights Reserved.

=head1 SEE ALSO

L<PureMessage::List>, L<PureMessage::MultiList>
