package PureMessage::MultiList::multifile;
use strict;
use Carp;
use PureMessage::List::Utils qw(list_absfile list_key2file list_file2key mkpath_dotdot path_rename);
use PureMessage::List::file;
use PureMessage::List::null;
use PureMessage::MultiList;
use base qw(PureMessage::MultiList);
use IO::Dir;

sub new {
    my $self = shift;
    my $class = ref($self) || $self;
    my $source = shift;
    my %list_conf = @_;
    my %conf = (
        _mf_dir => list_absfile($source),
        _mf_lists => {},
        _mf_conf => \%list_conf,
    );
    return bless \%conf, $class;
}

sub name { $_[0]->{_mf_conf}{name} }
sub description { $_[0]->{_mf_conf}{description} }

sub _create {
    my ($CLASS,$source,%options) = @_;
    die "Unknown source type '$1'" unless $source =~ /^([^:]+):(.+)$/ and $1 eq 'multifile';
    require PureMessage::List::Base::file;
    my $f = PureMessage::List::Base::file->_abs_file($2);
    die "Directory '$f' already exists." if -e $f and -d _;
    mkdir $f || die "Can't create directory '$f' for multifile";
    return 1; 
}

sub find_list {
    my $self = shift;
    my %opts = @_;
    my $key = $opts{key};
    croak "parameter 'key' required" unless defined $key;
    my $lists = $self->{_mf_lists};

    unless (exists $lists->{$key}) {
        my(@files) = (list_key2file($key, $self->{_mf_dir}));
        my(@e_files) = (grep(-f $_, list_key2file($key, $self->{_mf_dir})));
        if (@e_files) {
            # if the old filespec ($files[1]) is the only one that exists,
            # try to move it into the new directory format.

            if(@e_files == 1 && $e_files[0] eq $files[1]) {
                if(path_rename($files[1], $files[0], 1)) {
                    $e_files[0] = $files[0];
                } else {
                    warn qq{path_rename("$files[1]", "$files[0]") failed: "},
                        $! ? $! : $@;
                }
            }
            
            # if both old-location and new-location files were found,
            # something's gone wrong...
            
            if(@e_files == 2) {
                warn qq{MultiList inconsistency: both "$e_files[0]" and },
                    qq{"$e_files[1]" exist!};
            }
            
            $lists->{$key} = PureMessage::List::file->new($e_files[0],
                                                          %{$self->{_mf_conf}},
                                                          source => "file:$e_files[0]");
        } elsif ($opts{create_mode}) {
            mkpath_dotdot($files[0]);
            open(my $fh, ">$files[0]") or die "Can't open $files[0]: $!";
            close $fh or die "Can't write $files[0]: $!";
            $lists->{$key} = PureMessage::List::file->new($files[0],
                                                          %{$self->{_mf_conf}},
                                                          source => "file:$files[0]");
        } elsif ($opts{edit_mode}) {
            # don't cache in edit mode

            mkpath_dotdot($files[0]);
            return PureMessage::List::null->new($files[0],
                                                %{$self->{_mf_conf}},
                                                source => "file:$files[0]");
        } else {
            # not found and not in edit mode - just return.
            return undef;
        }
    }
    return $lists->{$key};
}

    
sub enumerate_keys {
    sub find_key_files {
        my $dir = shift;
        my $d = IO::Dir->new($dir) or die "$dir: $!";
        my(@contents) = grep(!/^\.(\.|~.*)?$/, $d->read);
        $d->close();
        my(@dirs) = grep(-d "$dir/$_", @contents);
        my(@files) = grep(-f "$dir/$_", @contents);
        return(@files, (map { find_key_files("$dir/$_") } (@dirs)));
    }

    my $self = shift;
    my $cb = shift;
    my %opts = @_;
    my $cb_arg = $opts{cb_arg};
    my @keys = map { list_file2key($_) } (find_key_files($self->{_mf_dir}));

    foreach my $key (@keys) {
        $cb->($cb_arg ? ($cb_arg, $key) : $key);
    }
}

sub refresh {
    my $self = shift;
    my $lists = $self->{_mf_lists};
    foreach (keys %$lists) {
        $lists->{$_}->refresh;
    }
}

sub _abs_file {
    my $class = shift;
    my $file = shift;
    return list_absfile($file);
}

1;
