package PureMessage::PerlStatus;

use strict;

sub make_report {
    my %opts = @_;
    my $print = delete $opts{print} || sub { print @_ };
    my $all = delete $opts{all};

    if ($all || $opts{inc}) {
	&$print("\n\n=== Loaded modules ===\n");
	for my $lib (sort keys %INC) {
	    $lib =~ s,/,::,g if $lib =~ s/\.pm$//;
	    &$print("  $lib\n");
	}
    }

    if ($all || $opts{symdump}) {
	&$print("\n\n=== Symbol table dump ===\n");
	eval {
	    require Devel::Symdump;
	};
	if ($@) {
	    &$print("  *** Devel::Symdump module not installed ***\n");
	}
	else {
	    my $dump = Devel::Symdump->rnew->as_string;
	    $dump =~ s/^/  /mg;
	    &$print($dump);
	}
    }

    if ($all || $opts{lexinfo}) {
	&$print("\n\n=== Lex info ===\n");
	eval {
	    require B::LexInfo;
	};
	if ($@) {
	    &$print("  *** B::LexInfo module not installed ***\n");
	}
	else {
	    my @pkg = packages();
	    my $lexi = B::LexInfo->new;
	    for my $pkg (@pkg) {
		my $info = $lexi->stash_cvlexinfo($pkg);
		require Data::Dump;
		my $dump = "$pkg " . Data::Dump::dump($info) . "\n";
		&$print($dump);
	    }
	}
    }

    if ($all || $opts{size}) {
	&$print("\n\n=== Memory usage ===\n\n");
	eval {
	    require B::TerseSize;
	};
	if ($@) {
	    &$print("  *** B::TerseSize module not installed ***\n");
	}
	else {
	    report_sizes($print);
	}

    }
}


sub report_sizes {
    my $print = shift;
    my %pkg;

    for my $pkg (packages()) {
	next if $pkg =~ /^B($|::)/;
	next if $pkg eq __PACKAGE__;
	next if $pkg eq "Apache::Status";
	$pkg{$pkg} = [reverse B::TerseSize::package_size($pkg)];
    }

    my @pkg_bysize = sort { $pkg{$b}[0] <=> $pkg{$a}[0] } keys %pkg;

    # Calculate totals;
    my $totsize = 0;
    my $totop = 0;
    for my $pkg (@pkg_bysize) {
	$totsize += $pkg{$pkg}[0];
	$totop += $pkg{$pkg}[1];
    }

    &$print("PACKAGES\n");
    &$print("--------------------------------------------------------\n");
    for my $pkg (@pkg_bysize) {
	&$print(sprintf "%5.1f%% %8d %6d %s\n", $pkg{$pkg}[0]/$totsize*100, $pkg{$pkg}[0], $pkg{$pkg}[1], $pkg);
    }
    &$print("------ -------- ------ ---------------------------------\n");
    &$print(sprintf "%5.1f%% %8d %6d %s\n", 100, $totsize, $totop, "TOTAL");
    &$print("\n");

    for my $pkg (@pkg_bysize) {
	my $info = $pkg{$pkg}[2];
	next unless %$info;
	&$print("\npackage $pkg\n");
	&$print("--------------------------------------------------------\n");
	#require Data::Dump; print Data::Dump::dump($info), "\n";
	my @bysize = sort { $info->{$b}{size} <=> $info->{$a}{size} } keys %$info;
	for my $f (@bysize) {
	    &$print(sprintf "%5.1f%% %8d %6d %s\n", $info->{$f}{size}/$pkg{$pkg}[0]*100, $info->{$f}{size}, $info->{$f}{count} || 0, $f);
	}
	&$print("------ -------- ------ ---------------------------------------\n");
	&$print(sprintf "%5.1f%% %8d %6d %s\n", 100, $pkg{$pkg}[0], $pkg{$pkg}[1], "TOTAL");
    }
}

sub packages {
    # based on Devel::Symdump
    my @todo = (shift || "main");
    my @pkg = @todo;
    while (@todo) {
	my $pkg = shift @todo;
	no strict 'refs';
	while (my($key, $val) = each(%{*{"$pkg\::"}})) {
	    local(*ENTRY) = $val;
	    if (defined($val) && defined(*ENTRY{HASH}) && $key =~ /::$/) {
		my $p = $pkg ne "main" ? "$pkg\::" : "";
		($p .= $key) =~ s/::$//;
		next if $p eq "main";    # avoid recusion
		next if $p eq "<none>";  # what is this?
		push(@pkg, $p);
		push(@todo, $p);
	    }
	}
    }
    @pkg;
}

1;
