package PureMessage::Service::Config;
use strict;
use PureMessage::Config qw(expand_filename);
use ActiveState::Config qw(:path);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = qw( %config %service_config &stale &is_stale);
our %config;
our $object;

our $service_config; # alias that doesn't clash with other %configs
*service_config =\%config;



sub refresh() {
    require PureMessage::Config::Validator;

    $object = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Config::Validator->define_core_validators($object);
    PureMessage::Service::Config->validate($object);
    $object->parse_file(expand_filename("etc/service.conf"));
    %config = $object->as_hash;
    foreach (qw( log_dir)) {
      $config{$_} = $PureMessage::Config::config{$_};
    }
}

refresh();

=item edit()

 Will open this config as a writeable  ActiveState::Config object
 and return it. This can die when configuration file has invalid
 syntax.

=cut

sub edit {
     my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
     PureMessage::Config::Validator->define_core_validators($p);
     PureMessage::Service::Config->validate($p);
     $p->parse_file(expand_filename("etc/service.conf"),writable => 1);
     return $p;
}

# mark this services config as stale as of C<time>
sub stale {
    my ($service,$name) = @_;
    $service = lc $service;
    $name = lc $name;
    my $p = edit();
    warn "Invalid arguments to stale" unless $service;
    return unless $service;
    my $path = "service#$service";
    $path .= $name ? ".name#$name.stale" : ".stale";
    path_value($p->root,$path,scalar(time));
    $p->save();
    refresh();
}

# get the current value of stale, or undef
sub is_stale {
    my ($service,$name) = @_;
    $service = lc $service;
    $name = lc $name;
    if ($name) {
       return $config{service}->{$service}->{name}->{$name}->{stale};
    } 
    else {
       my $gr = $config{service}->{$service}->{stale};
       foreach my $key (keys %{$config{service}->{$service}->{name}} ) {
          $gr = $config{service}->{$service}->{name}->{$key}->{stale} if $config{service}->{$service}->{name}->{$key}->{stale} > $gr; 
       }
       return $gr;
    }
}

sub validate {
    my ($pkg, $p) = @_;
    $p->typedecl('service#*.stale','#int');
    $p->typedecl('service#*.name#*.stale','#int');
    
    $pkg->include_sub_validators($p);
}

1;
