package PureMessage::UTF8::Config;

use strict;
use warnings;

use base qw(PureMessage::Config::Validator Exporter);
use PureMessage::Config qw(expand_filename);

our @EXPORT_OK = qw(%utf8_config);
our %utf8_config;

refresh();

sub refresh {
    require PureMessage::Config::Validator;

    my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);

    PureMessage::Config::Validator->define_core_validators($p);
    PureMessage::UTF8::Config->validate($p);
    $p->parse_file(expand_filename('etc/utf8.conf'));

    %utf8_config = $p->as_hash;
}

sub validate {
    my ($pkg, $p) = @_;

    my %conf = (
    'encoding#options.iconv_enabled' => ['#bool', 0],
	'encoding#guess.enabled'	=> ['#bool', 0],
	'encoding#guess.size'		=> ['#int', 256],
	'encoding#guess.threshold'	=> ['#num', 0.98],
	'encoding#guess.banner'		=> ['#str', undef],
	'encoding#guess.default'	=> ['#str', undef],
	'encoding#guess.replace_header'	=> ['#str', 'rfc2047'],
	'encoding#guess.*'		=> ['#bool', 0],
    );

    foreach my $k (keys %conf) {
	my ($t, $d) = @{$conf{$k}};

	$p->typedecl($k, $t) if defined $t;
	$p->default($k, $d) if defined $d;
    }
}

1;
