package PureMessage::Util::Email;
use strict;
use warnings;
use base qw(Exporter);
our @EXPORT_OK = qw(is_valid_local_email_part is_valid_email);

# Code by David T. McWherter <cache@cs.cmu.edu> from
# http://www.cs.cmu.edu/~cache/email/:
#
# This code handles non-obsolete address-bits without comments and without
# random (though legal) whitespace. is_valid_local_email_part takes the
# stuff before the '@' and tells you if it is valid. is_valid_email will
# validate an entire email address.

sub is_valid_local_email_part ($) {
    my ($addr) = @_;
    my $atext = qr/[A-Za-z0-9\!\#\$\%\&\'\*\+\-\/\=\?\^\_\`\{\|\+\~]/;
    my $dot_atom_text = qr/$atext+(\.$atext+)*/;

    my $no_ws_ctl_char = qr/[\x01-\x08\x0b\x0c\x0e-\x1f\x7f]/;
    my $qtext_char = qr/([\x21\x23-\x5b\x5d-\x7e]|$no_ws_ctl_char)/;
    my $text = qr/[\x01-\x09\x0b\x0c\x0e-\x7f]/;
    my $qtext = qr/($qtext_char|\\$text)*/;
    my $quoted_string = qr/"$qtext"/;

    if ($addr =~ /^($dot_atom_text|$quoted_string)$/) {
        return 1;
    } else {
        return 0;
    }
}

sub is_valid_email ($) {
    my ($addr) = @_;
    my $atext = qr/[A-Za-z0-9\!\#\$\%\&\'\*\+\-\/\=\?\^\_\`\{\|\+\~]/;
    my $dot_atom_text = qr/$atext+(\.$atext+)*/;

    my $no_ws_ctl_char = qr/[\x01-\x08\x0b\x0c\x0e-\x1f\x7f]/;
    my $qtext_char = qr/([\x21\x23-\x5b\x5d-\x7e]|$no_ws_ctl_char)/;
    my $text = qr/[\x01-\x09\x0b\x0c\x0e-\x7f]/;
    my $qtext = qr/($qtext_char|\\$text)*/;
    my $quoted_string = qr/"$qtext"/;

    my $quotedpair = qr/\\$text/;
    my $dtext = qr/[\x21-\x5a\x5e-\x7e\x01-\x08\x0b\x0c\x0e-\x1f\x7f]/;
    my $dcontent = qr/($dtext|$quotedpair)/;
    my $domain_literal = qr/\[(${dcontent})*\]/;

    if ($addr =~ /^($dot_atom_text|$quoted_string)\@($dot_atom_text|$domain_literal)$/) {
        return 1;
    } else {
        return 0;
    }
}

1;
