package pmx_user;
use base 'Exporter';

sub import {
    my ($self, @args) = @_;
    (my $pg = $0) =~ s,.*/,,;
    for my $arg (@args) {
	if ($arg eq 'strict') {
	    require PureMessage::Config;
	    if ($> != $PureMessage::Config::config{user}) {
		print STDERR "The $pg program must only be run as the ",
		    (getpwuid($PureMessage::Config::config{user}))[0],
		    " user.\n";
		exit(255);
	    }
	}
	elsif ($arg eq 'switch') {
	    set_uid_gid() if $> == 0;
	}
	else {
	    die "Unknown argument '$arg' passed to to pmx_user";
	}
    }
    if ($> == 0) {
	print STDERR "The $pg program cannot be run as the root user.\n";
	exit(255);
    }
}

sub set_uid_gid {
    our $PREFIX;
    require PureMessage::Version;
    PureMessage::Version->import('$PREFIX');

    my $pmx = "$PREFIX/bin/pmx";
    # we don't use @config{user,group} here, because we don't want
    # to load PureMessage::Config as root
    my ($u, $g) = (stat $pmx)[4,5];
    die "Can't stat $pmx: $!" unless defined $u;

    # set GID
    if ($) != $g) {
        $) = $g;
        die "Can't set effective GID to $g: $!"
            unless $) == $g;
        $( = $g;
        die "Can't set real GID to $g: $!"
            unless $( == $g;
    }

    # set UID
    if ($> != $u) {
        # Must set both in one statement to avoid taint problems.
        ($<, $>) = ($u, $u);
        die "Can't set real UID to $u: $!"
            unless $< == $u;
        die "Can't set effective UID to $u: $!"
            unless $> == $u;
    }
}

1;

__END__

=head1 NAME

pmx_user - disallow running as root

=head1 SYNOPSIS

    use pmx_user;
    use pmx_user 'strict';
    use pmx_user 'switch';

=head1 DESCRIPTION

This module acts as a pragma and can be used by programs that want to
prevent root running them directly.  When the module is loaded with
the C<use> statement it will check the current UID and will croak with
an error message when it is run as the root user.  PureMessage command
line programs should always use this unless they contain code so that
they behave correctly when run as root.

If the argument 'strict' is passed with the use statement then the
program will die unless it is actually running as the PureMessage
user.

If the argument 'switch' is passed with the use statement and the
program is being run as the root user, it will attempt to switch
to the PureMessage user and group IDs.  Execution will fail if
the attempt to assume the identity of the PureMessage user was
unsuccessful.  The 'switch' argument is ignored when not running
as 'root'.

=cut
